@@ -14,6 +14,8 @@ from compliance_checker.base import (BaseCheck, BaseNCCheck, check_has,
 from compliance_checker.cf.util import _possiblexunits, _possibleyunits
 from compliance_checker.util import datetime_is_iso, dateparse
 from compliance_checker import cfutil
+from compliance_checker.util import kvp_convert
+from functools import partial
 import pendulum
 from pygeoif import from_wkt
 
@@ -223,11 +225,23 @@ class ACDDBaseCheck(BaseCheck):
 
         :param netCDF4.Dataset ds: An open netCDF dataset
         '''
-        if not (hasattr(ds, 'geospatial_lat_min') and hasattr(ds, 'geospatial_lat_max')):
-            return
 
-        lat_min = ds.geospatial_lat_min
-        lat_max = ds.geospatial_lat_max
+        if not (hasattr(ds, 'geospatial_lat_min') or hasattr(ds, 'geospatial_lat_max')):
+            return Result(BaseCheck.MEDIUM,
+                          False,
+                          'geospatial_lat_extents_match',
+                          ['geospatial_lat_min/max attribute not found, CF-1.6 spec chapter 4.1'])
+
+        try: # type cast
+            lat_min = float(ds.geospatial_lat_min)
+            lat_max = float(ds.geospatial_lat_max)
+        except ValueError:
+            return Result(BaseCheck.MEDIUM,
+                          False,
+                          'geospatial_lat_extents_match',
+                          ['Could not convert one of geospatial_lat_min ({}) or max ({}) to float see CF-1.6 spec chapter 4.1'
+                          ''.format(ds.geospatial_lat_min, ds.geospatial_lat_max)])
+
 
         # identify lat var(s) as per CF 4.1
         lat_vars = {}       # var -> number of criteria passed
@@ -289,11 +303,22 @@ class ACDDBaseCheck(BaseCheck):
 
         :param netCDF4.Dataset ds: An open netCDF dataset
         '''
-        if not (hasattr(ds, 'geospatial_lon_min') and hasattr(ds, 'geospatial_lon_max')):
-            return
 
-        lon_min = ds.geospatial_lon_min
-        lon_max = ds.geospatial_lon_max
+        if not (hasattr(ds, 'geospatial_lon_min') and hasattr(ds, 'geospatial_lon_max')):
+            return Result(BaseCheck.MEDIUM,
+                          False,
+                          'geospatial_lon_extents_match',
+                          ['geospatial_lon_min/max attribute not found, CF-1.6 spec chapter 4.1'])
+
+        try: # type cast
+            lon_min = float(ds.geospatial_lon_min)
+            lon_max = float(ds.geospatial_lon_max)
+        except ValueError:
+            return Result(BaseCheck.MEDIUM,
+                          False,
+                          'geospatial_lon_extents_match',
+                          ['Could not convert one of geospatial_lon_min ({}) or max ({}) to float see CF-1.6 spec chapter 4.1'
+                          ''.format(ds.geospatial_lon_min, ds.geospatial_lon_max)])
 
         # identify lon var(s) as per CF 4.2
         lon_vars = {}       # var -> number of criteria passed
@@ -363,8 +388,11 @@ class ACDDBaseCheck(BaseCheck):
             from_wkt(ds.geospatial_bounds)
         except AttributeError:
             return ratable_result(False,
-                                  "Global Attributes", # grouped with Globals
-                                  ['Could not parse WKT, possible bad value for WKT'])
+                                  "Global Attributes",  # grouped with Globals
+                                  [('Could not parse WKT from geospatial_bounds,'
+                                    ' possible bad value: \"{}\"'.format(ds.geospatial_bounds))],
+                                  variable_name='geospatial_bounds'
+                                  )
         # parsed OK
         else:
             return ratable_result(True, "Global Attributes", tuple())
@@ -377,30 +405,48 @@ class ACDDBaseCheck(BaseCheck):
         :param netCDF4.Dataset ds: An open netCDF dataset
         :param str z_variable: Name of the variable representing the Z-Axis
         '''
-        vert_min = ds.geospatial_vertical_min
-        vert_max = ds.geospatial_vertical_max
         msgs = []
         total = 2
+        try:
+            vert_min = float(ds.geospatial_vertical_min)
+        except ValueError:
+            msgs.append('geospatial_vertical_min cannot be cast to float')
+
+        try:
+            vert_max = float(ds.geospatial_vertical_max)
+        except ValueError:
+            msgs.append('geospatial_vertical_max cannot be cast to float')
+        if len(msgs) > 0:
+            return Result(BaseCheck.MEDIUM, (0, total),
+                          'geospatial_vertical_extents_match', msgs)
 
         zvalue = ds.variables[z_variable][:]
         # If the array has fill values, which is allowed in the case of point
         # features
         if hasattr(zvalue, 'mask'):
             zvalue = zvalue[~zvalue.mask]
-        zmin = zvalue.min()
-        zmax = zvalue.max()
-        if not np.isclose(vert_min, zmin):
-            msgs.append("geospatial_vertical_min != min(%s) values, %s != %s" % (
-                z_variable,
-                vert_min,
-                zmin
-            ))
-        if not np.isclose(vert_max, zmax):
-            msgs.append("geospatial_vertical_max != max(%s) values, %s != %s" % (
-                z_variable,
-                vert_min,
-                zmax
-            ))
+
+        if zvalue.size == 0:
+            msgs.append("Cannot compare geospatial vertical extents "
+                        "against min/max of data, as non-masked data "
+                        "length is zero")
+            return Result(BaseCheck.MEDIUM, (0, total),
+                          'geospatial_vertical_extents_match', msgs)
+        else:
+            zmin = zvalue.min()
+            zmax = zvalue.max()
+            if not np.isclose(vert_min, zmin):
+                msgs.append("geospatial_vertical_min != min(%s) values, %s != %s" % (
+                    z_variable,
+                    vert_min,
+                    zmin
+                ))
+            if not np.isclose(vert_max, zmax):
+                msgs.append("geospatial_vertical_max != max(%s) values, %s != %s" % (
+                    z_variable,
+                    vert_min,
+                    zmax
+                ))
 
         return Result(BaseCheck.MEDIUM,
                       (total - len(msgs), total),
@@ -611,6 +657,28 @@ class ACDD1_3Check(ACDDNCCheck):
             'publisher_institution',
         ])
 
+        # overrride the ISO date checks in
+        def _check_attr_is_iso_date(attr, ds):
+            result_name = "{}_is_iso".format(attr)
+            if not hasattr(ds, attr):
+                return ratable_result((0,2), result_name,
+                                      ["Attr {} is not present".format(attr)])
+            else:
+                iso_check, msgs = datetime_is_iso(getattr(ds, attr))
+                return ratable_result((1 + iso_check, 2),
+                                      result_name, msgs)
+
+        # run ISO 8601 date checks against the date_created, date_issued,
+        # date_modified, and date_metadata_modified global attributes
+        self.rec_atts = kvp_convert(self.rec_atts)
+        self.rec_atts['date_created'] = partial(_check_attr_is_iso_date,
+                                                'date_created')
+        self.sug_atts = kvp_convert(self.sug_atts)
+        for k in ("date_{}".format(suffix) for suffix in
+                  ('issued', 'modified', 'metadata_modified')):
+
+            self.sug_atts[k] = partial(_check_attr_is_iso_date, k)
+
     def check_metadata_link(self, ds):
         '''
         Checks if metadata link is formed in a rational manner
@@ -626,39 +694,6 @@ class ACDD1_3Check(ACDDNCCheck):
         valid_link = (len(msgs) == 0)
         return Result(BaseCheck.LOW, valid_link, 'metadata_link_valid', msgs)
 
-    def check_date_modified_is_iso(self, ds):
-        '''
-        Checks if date modified field is ISO compliant
-
-        :param netCDF4.Dataset ds: An open netCDF dataset
-        '''
-        if not hasattr(ds, u'date_modified'):
-            return
-        date_modified_check, msgs = datetime_is_iso(getattr(ds, u'date_modified'))
-        return Result(BaseCheck.MEDIUM, date_modified_check, 'date_modified_is_iso', msgs)
-
-    def check_date_issued_is_iso(self, ds):
-        '''
-        Checks if date issued field is ISO compliant
-
-        :param netCDF4.Dataset ds: An open netCDF dataset
-        '''
-        if not hasattr(ds, u'date_issued'):
-            return
-        date_issued_check, msgs = datetime_is_iso(getattr(ds, u'date_issued'))
-        return Result(BaseCheck.MEDIUM, date_issued_check, 'date_issued_is_iso', msgs)
-
-    def check_date_metadata_modified_is_iso(self, ds):
-        '''
-        Checks if date metadata modified field is ISO compliant
-
-        :param netCDF4.Dataset ds: An open netCDF dataset
-        '''
-        if not hasattr(ds, u'date_metadata_modified'):
-            return
-        date_metadata_modified_check, msgs = datetime_is_iso(getattr(ds, u'date_metadata_modified'))
-        return Result(BaseCheck.MEDIUM, date_metadata_modified_check, 'date_metadata_modified_is_iso', msgs)
-
     def check_id_has_no_blanks(self, ds):
         '''
         Check if there are blanks in the id field
@@ -673,18 +708,6 @@ class ACDD1_3Check(ACDDNCCheck):
         else:
             return Result(BaseCheck.MEDIUM, True, 'no_blanks_in_id', msgs=[])
 
-    def check_date_created(self, ds):
-        '''
-        Check if date created is ISO-8601
-
-        :param netCDF4.Dataset ds: An open netCDF dataset
-        '''
-        if not hasattr(ds, u'date_created'):
-            return
-        date_created_check, msgs = datetime_is_iso(getattr(ds, u'date_created'))
-        return Result(BaseCheck.MEDIUM, date_created_check,
-                      'date_created_is_iso', msgs)
-
     def check_var_coverage_content_type(self, ds):
         '''
         Check coverage content type against valid ISO-19115-1 codes
@@ -698,7 +721,7 @@ class ACDD1_3Check(ACDDNCCheck):
                             'coverage_content_type', None)
             check = ctype is not None
             if not check:
-                msgs.append("coverage_content")
+                msgs.append("coverage_content_type")
                 results.append(Result(BaseCheck.HIGH, check,
                                       self._var_header.format(variable), msgs))
                 continue
