@@ -14,6 +14,7 @@ from netCDF4 import Dataset
 from lxml import etree as ET
 from distutils.version import StrictVersion
 from compliance_checker.base import fix_return_value, Result, GenericFile
+from compliance_checker.cf.cf import CFBaseCheck
 from owslib.sos import SensorObservationService
 from owslib.swe.sensor.sml import SensorML
 from compliance_checker.protocols import opendap, netcdf, cdl
@@ -28,7 +29,10 @@ except ImportError:
 from datetime import datetime
 import requests
 import codecs
+import re
+import textwrap
 from pkg_resources import working_set
+import six
 
 
 # Ensure output is encoded as Unicode when checker output is redirected or piped
@@ -37,6 +41,20 @@ if sys.stdout.encoding is None:
 if sys.stderr.encoding is None:
     sys.stderr = codecs.getwriter('utf8')(sys.stderr)
 
+def extract_docstring_summary(docstring):
+    """
+    Returns a dedented docstring without parameter information
+    :param docstring: A docstring
+    :type docstring: str
+    :returns: str
+    """
+    # return a dedented, then indented two spaces docstring with leading and
+    # trailing whitespace removed.
+    return re.sub(r'^(?=.)', '  ',
+                  textwrap.dedent(re.split(r'\n\s*:\w', docstring,
+                                           flags=re.MULTILINE)[0]).strip(),
+                  flags=re.MULTILINE)
+
 class CheckSuite(object):
 
     checkers = {}       # Base dict of checker names to BaseCheck derived types, override this in your CheckSuite implementation
@@ -58,6 +76,45 @@ class CheckSuite(object):
 
         return cls.suite_generators
 
+    def _print_suites(self, verbose=0):
+        """
+        Prints out available check suites.  If the verbose argument is True,
+        includes the internal module version number of the check and also displays
+        "latest" meta-versions.
+        :param check_suite: Check suite object
+        :param verbose: Integer indicating whether to print verbose output
+        :type verbose: int
+        """
+        for checker in sorted(self.checkers.keys()):
+            version = getattr(self.checkers[checker],
+                            '_cc_checker_version', "???")
+            if verbose > 0:
+                print(" - {} (v{})".format(checker, version))
+            elif ':' in checker and not checker.endswith(':latest'):  # Skip the "latest" output
+                print(" - {}".format(checker))
+
+    def _print_checker(self, checker_obj):
+        """
+        Prints each available check and a description with an abridged
+        docstring for a given checker object
+        :param checker_obj: Checker object on which to operate
+        :type checker_obj: subclass of compliance_checker.base.BaseChecker
+        """
+
+        check_functions = self._get_checks(checker_obj,
+                                           defaultdict(lambda: None))
+        for c, _ in check_functions:
+            print("- {}".format(c.__name__))
+            if c.__doc__ is not None:
+                # necessary for unicode characters in docstrings under Python
+                # 2, aside from converting everything over to u'' strings
+                try:
+                    u_doc = six.u(c.__doc__)
+                except TypeError:
+                    u_doc = c.__doc__
+
+                print("\n{}\n".format(extract_docstring_summary(u_doc)))
+
     @classmethod
     def add_plugin_args(cls, parser):
         """
@@ -83,22 +140,42 @@ class CheckSuite(object):
         Helper method to retrieve all sub checker classes derived from various
         base classes.
         """
-        for x in working_set.iter_entry_points('compliance_checker.suites'):
+        cls._load_checkers(working_set
+                           .iter_entry_points('compliance_checker.suites'))
+
+    @classmethod
+    def _load_checkers(cls, checkers):
+        """
+        Loads up checkers in an iterable into the class checkers dict
+        :param checkers: An iterable containing the checker objects
+        """
+
+        for c in checkers:
             try:
-                xl = x.resolve()
-                cls.checkers[':'.join((xl._cc_spec, xl._cc_spec_version))] = xl
-            # TODO: remove this once all checkers move over to the new
-            #       _cc_spec, _cc_spec_version
-            except AttributeError:
-                # if there are versioned classes, it will get overwritten by the
-                # latest version later.  If there are not, it will be assigned
-                # the checker as the main class
-                # TODO: nix name attribute in plugins.  Keeping in for now
-                #       to provide backwards compatibility
-                cls.checkers[getattr(xl, 'name', None) or xl._cc_spec] = xl
+                check_obj = c.resolve()
+                if (hasattr(check_obj, '_cc_spec') and
+                    hasattr(check_obj, '_cc_spec_version')):
+                    check_version_str = ':'.join((check_obj._cc_spec,
+                                                  check_obj._cc_spec_version))
+                    cls.checkers[check_version_str] = check_obj
+                # TODO: remove this once all checkers move over to the new
+                #       _cc_spec, _cc_spec_version
+                else:
+                    # if _cc_spec and _cc_spec_version attributes aren't
+                    # present, fall back to using name attribute
+                    checker_name = (getattr(check_obj, 'name', None) or
+                                    getattr(check_obj, '_cc_spec', None))
+                    warnings.warn('Checker for {} should implement both '
+                                '"_cc_spec" and "_cc_spec_version" '
+                                'attributes. "name" attribute is deprecated. '
+                                'Assuming checker is latest version.',
+                                DeprecationWarning)
+                    # append "unknown" to version string since no versioning
+                    # info was provided
+                    cls.checkers["{}:unknown".format(checker_name)] = check_obj
 
             except Exception as e:
-                print("Could not load", x, ":", e, file=sys.stderr)
+                print("Could not load", c, ":", e, file=sys.stderr)
         # find the latest version of versioned checkers and set that as the
         # default checker for compliance checker if no version is specified
         ver_checkers = sorted([c.split(':', 1) for c
@@ -120,10 +197,10 @@ class CheckSuite(object):
         Helper method to retreive check methods from a Checker class.  Excludes
         any checks in `skip_checks`.
 
-        The name of the methods in the Checker class should start with "check_" for this
-        method to find them.
+        The name of the methods in the Checker class should start with "check_"
+        for this method to find them.
         """
-        meths = inspect.getmembers(checkclass, inspect.ismethod)
+        meths = inspect.getmembers(checkclass, inspect.isroutine)
         # return all check methods not among the skipped checks
         returned_checks = []
         for fn_name, fn_obj in meths:
@@ -142,9 +219,15 @@ class CheckSuite(object):
         @return list: list of Result objects
         """
         val = check_method(ds)
-        if isinstance(val, list):
+        if hasattr(val, '__iter__'):
+            # Handle OrderedDict when we need to modify results in a superclass
+            # i.e. some checks in CF 1.7 which extend CF 1.6 behaviors
+            if isinstance(val, dict):
+                val_iter = val.values()
+            else:
+                val_iter = val
             check_val = []
-            for v in val:
+            for v in val_iter:
                 res = fix_return_value(v, check_method.__func__.__name__,
                                        check_method, check_method.__self__)
                 if max_level is None or res.weight > max_level:
@@ -194,7 +277,7 @@ class CheckSuite(object):
         args = [(name, self.checkers[name]) for name in checker_names if name in self.checkers]
         valid = []
 
-        all_checked = set([a[1] for a in args])  # only class types
+        all_checked = set(a[1] for a in args)  # only class types
         checker_queue = set(args)
         while len(checker_queue):
             name, a = checker_queue.pop()
@@ -203,11 +286,12 @@ class CheckSuite(object):
             if type(ds) in a().supported_ds:
                 valid.append((name, a))
 
-            # add any subclasses of the checker class
-            for subc in a.__subclasses__():
-                if subc not in all_checked:
-                    all_checked.add(subc)
-                    checker_queue.add((name, subc))
+            # add subclasses of SOS checks
+            if "ioos_sos" in name:
+                for subc in a.__subclasses__():
+                    if subc not in all_checked:
+                        all_checked.add(subc)
+                        checker_queue.add((name, subc))
 
         return valid
 
@@ -586,7 +670,9 @@ class CheckSuite(object):
     def generate_dataset(self, cdl_path):
         '''
         Use ncgen to generate a netCDF file from a .cdl file
-        Returns the path to the generated netcdf file
+        Returns the path to the generated netcdf file. If ncgen fails, uses
+        sys.exit(1) to terminate program so a long stack trace is not reported
+        to the user.
 
         :param str cdl_path: Absolute path to cdl file that is used to generate netCDF file
         '''
@@ -594,7 +680,10 @@ class CheckSuite(object):
             ds_str = cdl_path.replace('.cdl', '.nc')
         else:
             ds_str = cdl_path + '.nc'
-        subprocess.call(['ncgen', '-o', ds_str, cdl_path])
+        returncode = subprocess.call(['ncgen', '-o', ds_str, cdl_path])
+        if returncode != 0:
+            print("Error generating dataset from .cdl")
+            sys.exit(1)
         return ds_str
 
     def load_dataset(self, ds_str):
@@ -662,7 +751,6 @@ class CheckSuite(object):
         @param list raw_scores: list of raw scores (Result objects)
         """
 
-        # BEGIN INTERNAL FUNCS ########################################
         def trim_groups(r):
             if isinstance(r.name, tuple) or isinstance(r.name, list):
                 new_name = r.name[1:]
